@@ -6,10 +6,8 @@ from __future__ import print_function, division, absolute_import
 import os
 
 import reVX.plexos as reV_plexos
-import reVX.reeds as rev_reeds
-import reVX.rpm as rev_rpm
-import reVX.wind_dirs as wind_dirs
-import reVX.wind_setbacks as wind_setbacks
+import reVX.reeds as reV_reeds
+import reVX.rpm as reV_rpm
 from reVX.version import __version__
 
 __author__ = """Michael Rossol"""
