@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
 """
-Module to compute least cost xmission paths, distances, and costs one or
+Module to compute least cost xmission paths, distances, AND costs for one or
 more SC points
 """
-from concurrent.futures import as_completed
 import geopandas as gpd
 import json
 import logging
 import numpy as np
 import os
 import pandas as pd
-from pyproj.crs import CRS
 import rasterio
+from pyproj.crs import CRS
 from scipy.spatial import cKDTree
 from shapely.geometry import Point
+from concurrent.futures import as_completed
 import time
 
 from reV.handlers.exclusions import ExclusionLayers
@@ -223,7 +223,8 @@ class LeastCostXmission(LeastCostPaths):
         sc_points : gpd.GeoDataFrame
             SC points
         """
-        logger.debug('Loading Supply Curve Points')
+        logger.debug('Loading Supply Curve Points for res: {}'
+                     .format(resolution))
         sce = SupplyCurveExtent(cost_fpath, resolution=resolution)
         sc_points = sce.points.rename(columns={'row_ind': 'sc_row_ind',
                                                'col_ind': 'sc_col_ind'})
@@ -339,7 +340,7 @@ class LeastCostXmission(LeastCostPaths):
         return sc_points, features, sub_lines_map, shape
 
     def _clip_to_sc_point(self, sc_point, tie_line_voltage, nn_sinks=2,
-                          clipping_buffer=1.05):
+                          clipping_buffer=1.05, radius=None):
         """
         Clip costs raster to AOI around SC point, and get substations,
         load centers, and sinks within the clipped region.
@@ -352,6 +353,8 @@ class LeastCostXmission(LeastCostPaths):
             Number of nearest neighbor sinks to clip to
         clipping_buffer : float, optional
             Buffer to increase clipping radius by, by default 1.05
+        radius : None | int, optional
+            Force clipping radius if set to an int
 
         Returns
         -------
@@ -362,14 +365,21 @@ class LeastCostXmission(LeastCostPaths):
             to SC point
         """
         logger.debug('Clipping features to sc_point {}'.format(sc_point.name))
-        if len(self.sink_coords) > 2:
-            row, col = sc_point[['row', 'col']].values
-            _, pos = self.sink_tree.query([row, col], k=nn_sinks)
-            radius = np.abs(self.sink_coords[pos] - np.array([row, col])).max()
-            radius = int(np.ceil(radius * clipping_buffer))
 
-            logger.debug('Radius to {} nearest sink is: {}'
-                         .format(nn_sinks, radius))
+        if len(self.sink_coords) > 2 or radius:
+            row, col = sc_point[['row', 'col']].values
+
+            if radius is None:
+                _, pos = self.sink_tree.query([row, col], k=nn_sinks)
+                radius = np.abs(self.sink_coords[pos] - np.array([row, col])
+                                ).max()
+                radius = int(np.ceil(radius * clipping_buffer))
+
+            if radius:
+                logger.debug('Using forced radius of {}'.format(radius))
+            else:
+                logger.debug('Radius to {} nearest sink is: {}'
+                             .format(nn_sinks, radius))
             row_min = max(row - radius, 0)
             row_max = min(row + radius, self._shape[0])
             col_min = max(col - radius, 0)
@@ -388,7 +398,6 @@ class LeastCostXmission(LeastCostPaths):
                          'radius {}'
                          .format(len(sc_features), radius))
         else:
-            radius = None
             sc_features = self.features.copy(deep=True)
 
         mask = self.features['max_volts'] >= tie_line_voltage
@@ -418,7 +427,8 @@ class LeastCostXmission(LeastCostPaths):
 
     def process_sc_points(self, capacity_class, sc_point_gids=None, nn_sinks=2,
                           clipping_buffer=1.05, barrier_mult=100,
-                          max_workers=None):
+                          max_workers=None, save_paths=False, radius=None,
+                          mp_delay=3, simplify_geo=None):
         """
         Compute Least Cost Tranmission for desired sc_points
 
@@ -428,7 +438,7 @@ class LeastCostXmission(LeastCostPaths):
             Capacity class of transmission features to connect supply curve
             points to
         sc_point_gids : list, optional
-            List of sc_point_gids to connect to, by default None
+            List of sc_point_gids to connect to, by default connect to all
         nn_sinks : int, optional
             Number of nearest neighbor sinks to use for clipping radius
             calculation, by default 2
@@ -440,10 +450,20 @@ class LeastCostXmission(LeastCostPaths):
         max_workers : int, optional
             Number of workers to use for processing, if 1 run in serial,
             if None use all available cores, by default None
+        save_paths : bool, optional
+            Flag to return least cost paths as a multi-line geometry,
+            by default False
+        radius : None | int, optional
+            Force clipping radius if set to an int
+        mp_delay : float, optional
+            Delay in seconds between starting multiprocess workers. Useful for
+            reducing memory spike at working startup.
+        simplify_geo : float | None, optional
+            If float, simplify geometries using this value
 
         Returns
         -------
-        least_costs : pandas.DataFrame
+        least_costs : pandas.DataFrame | gpd.GeoDataFrame
             Least cost connections between all supply curve points and the
             transmission features with the given capacity class that are within
             "nn_sink" nearest infinite sinks
@@ -454,66 +474,35 @@ class LeastCostXmission(LeastCostPaths):
             sc_point_gids = self.sc_points['sc_point_gid'].values
 
         tie_line_voltage = self._config.capacity_to_kv(capacity_class)
-        least_costs = []
+
         if max_workers > 1:
             logger.info('Computing Least Cost Transmission for SC points in '
                         'parallel on {} workers'.format(max_workers))
-            loggers = [__name__, 'reV', 'reVX']
-            with SpawnProcessPool(max_workers=max_workers,
-                                  loggers=loggers) as exe:
-                futures = []
-                for _, sc_point in self.sc_points.iterrows():
-                    gid = sc_point['sc_point_gid']
-                    if gid in sc_point_gids:
-                        sc_features, radius = self._clip_to_sc_point(
-                            sc_point, tie_line_voltage, nn_sinks=nn_sinks,
-                            clipping_buffer=clipping_buffer)
-
-                        future = exe.submit(TransCapCosts.run,
-                                            self._cost_fpath,
-                                            sc_point.copy(deep=True),
-                                            sc_features, capacity_class,
-                                            radius=radius,
-                                            xmission_config=self._config,
-                                            barrier_mult=barrier_mult,
-                                            min_line_length=self._min_line_len)
-                        futures.append(future)
-
-                for i, future in enumerate(as_completed(futures)):
-                    sc_costs = future.result()
-                    if sc_costs is not None:
-                        least_costs.append(sc_costs)
-
-                    logger.debug('SC point {} of {} complete!'
-                                 .format(i + 1, len(futures)))
-                    log_mem(logger)
-
+            least_costs = self._process_multi_core(
+                capacity_class,
+                tie_line_voltage,
+                sc_point_gids=sc_point_gids,
+                nn_sinks=nn_sinks,
+                clipping_buffer=clipping_buffer,
+                barrier_mult=barrier_mult,
+                save_paths=save_paths,
+                radius=radius,
+                mp_delay=mp_delay,
+                simplify_geo=simplify_geo,
+                max_workers=max_workers)
         else:
             logger.info('Computing Least Cost Transmission for SC points in '
                         'serial')
-            i = 1
-            for _, sc_point in self.sc_points.iterrows():
-                gid = sc_point['sc_point_gid']
-                if gid in sc_point_gids:
-                    sc_features, radius = self._clip_to_sc_point(
-                        sc_point, tie_line_voltage, nn_sinks=nn_sinks,
-                        clipping_buffer=clipping_buffer)
-
-                    sc_costs = TransCapCosts.run(
-                        self._cost_fpath, sc_point.copy(deep=True),
-                        sc_features, capacity_class,
-                        radius=radius,
-                        xmission_config=self._config,
-                        barrier_mult=barrier_mult,
-                        min_line_length=self._min_line_len)
-
-                    if sc_costs is not None:
-                        least_costs.append(sc_costs)
-
-                    logger.debug('SC point {} of {} complete!'
-                                 .format(i, len(sc_point_gids)))
-                    log_mem(logger)
-                    i += 1
+            least_costs = self._process_single_core(
+                capacity_class,
+                tie_line_voltage,
+                sc_point_gids=sc_point_gids,
+                nn_sinks=nn_sinks,
+                clipping_buffer=clipping_buffer,
+                barrier_mult=barrier_mult,
+                save_paths=save_paths,
+                radius=radius,
+                simplify_geo=simplify_geo)
 
         least_costs = pd.concat(least_costs).sort_values(['sc_point_gid',
                                                           'trans_gid'])
@@ -526,10 +515,168 @@ class LeastCostXmission(LeastCostPaths):
 
         return least_costs.reset_index(drop=True)
 
+    def _process_multi_core(self, capacity_class, tie_line_voltage,
+                            sc_point_gids=None, nn_sinks=2,
+                            clipping_buffer=1.05, barrier_mult=100,
+                            max_workers=2, save_paths=False, radius=None,
+                            mp_delay=3, simplify_geo=None):
+        """
+        Compute Least Cost Tranmission for desired sc_points using multiple
+        cores.
+
+        Parameters
+        ----------
+        capacity_class : str | int
+            Capacity class of transmission features to connect supply curve
+            points to
+        tie_line_voltage : int
+            Tie-line volatage (kV)
+        sc_point_gids : list, optional
+            List of sc_point_gids to connect to, by default connect to all
+        nn_sinks : int, optional
+            Number of nearest neighbor sinks to use for clipping radius
+            calculation, by default 2
+        clipping_buffer : float, optional
+            Buffer to expand clipping radius by, by default 1.05
+        barrier_mult : int, optional
+            Tranmission barrier multiplier, used when computing the least
+            cost tie-line path, by default 100
+        max_workers : int, optional
+            Number of workers to use for processing
+        save_paths : bool, optional
+            Flag to return least cost paths as a multi-line geometry,
+            by default False
+        radius : None | int, optional
+            Force clipping radius if set to an int
+        mp_delay : float, optional
+            Delay in seconds between starting multiprocess workers. Useful for
+            reducing memory spike at working startup.
+        simplify_geo : float | None, optional
+            If float, simplify geometries using this value
+
+        Returns
+        -------
+        least_costs : pandas.DataFrame | gpd.GeoDataFrame
+            Least cost connections between all supply curve points and the
+            transmission features with the given capacity class that are within
+            "nn_sink" nearest infinite sinks
+        """
+        least_costs = []
+        num_jobs = 0
+        loggers = [__name__, 'reV', 'reVX']
+        with SpawnProcessPool(max_workers=max_workers, loggers=loggers) as exe:
+            futures = []
+            for _, sc_point in self.sc_points.iterrows():
+                gid = sc_point['sc_point_gid']
+                if gid in sc_point_gids:
+                    sc_features, radius = self._clip_to_sc_point(
+                        sc_point, tie_line_voltage, nn_sinks=nn_sinks,
+                        clipping_buffer=clipping_buffer, radius=radius)
+
+                    future = exe.submit(TransCapCosts.run,
+                                        self._cost_fpath,
+                                        sc_point.copy(deep=True),
+                                        sc_features, capacity_class,
+                                        radius=radius,
+                                        xmission_config=self._config,
+                                        barrier_mult=barrier_mult,
+                                        min_line_length=self._min_line_len,
+                                        save_paths=save_paths,
+                                        simplify_geo=simplify_geo)
+                    futures.append(future)
+
+                    num_jobs += 1
+                    if num_jobs <= max_workers:
+                        time.sleep(mp_delay)
+
+            logger.debug('Completed kicking off {} jobs for {} workers.'
+                         .format(num_jobs, max_workers))
+            for i, future in enumerate(as_completed(futures)):
+                sc_costs = future.result()
+                if sc_costs is not None:
+                    least_costs.append(sc_costs)
+
+                logger.debug('SC point {} of {} complete!'
+                             .format(i + 1, len(futures)))
+                log_mem(logger)
+
+        return least_costs
+
+    def _process_single_core(self, capacity_class, tie_line_voltage,
+                             sc_point_gids=None, nn_sinks=2,
+                             clipping_buffer=1.05, barrier_mult=100,
+                             save_paths=False, radius=None,
+                             simplify_geo=None):
+        """
+        Compute Least Cost Tranmission for desired sc_points with a single
+        core.
+
+        Parameters
+        ----------
+        capacity_class : str | int
+            Capacity class of transmission features to connect supply curve
+            points to
+        tie_line_voltage : int
+            Tie-line volatage (kV)
+        sc_point_gids : list, optional
+            List of sc_point_gids to connect to, by default connect to all
+        nn_sinks : int, optional
+            Number of nearest neighbor sinks to use for clipping radius
+            calculation, by default 2
+        clipping_buffer : float, optional
+            Buffer to expand clipping radius by, by default 1.05
+        barrier_mult : int, optional
+            Tranmission barrier multiplier, used when computing the least
+            cost tie-line path, by default 100
+        save_paths : bool, optional
+            Flag to return least cost paths as a multi-line geometry,
+            by default False
+        radius : None | int, optional
+            Force clipping radius if set to an int
+        simplify_geo : float | None, optional
+            If float, simplify geometries using this value
+
+        Returns
+        -------
+        least_costs : pandas.DataFrame | gpd.GeoDataFrame
+            Least cost connections between all supply curve points and the
+            transmission features with the given capacity class that are within
+            "nn_sink" nearest infinite sinks
+        """
+        least_costs = []
+        i = 1
+        for _, sc_point in self.sc_points.iterrows():
+            gid = sc_point['sc_point_gid']
+            if gid in sc_point_gids:
+                sc_features, radius = self._clip_to_sc_point(
+                    sc_point, tie_line_voltage, nn_sinks=nn_sinks,
+                    clipping_buffer=clipping_buffer, radius=radius)
+
+                sc_costs = TransCapCosts.run(
+                    self._cost_fpath,
+                    sc_point.copy(deep=True),
+                    sc_features, capacity_class,
+                    radius=radius,
+                    xmission_config=self._config,
+                    barrier_mult=barrier_mult,
+                    min_line_length=self._min_line_len,
+                    save_paths=save_paths,
+                    simplify_geo=simplify_geo)
+
+                if sc_costs is not None:
+                    least_costs.append(sc_costs)
+
+                logger.debug('SC point {} of {} complete!'
+                             .format(i, len(sc_point_gids)))
+                log_mem(logger)
+                i += 1
+        return least_costs
+
     @classmethod
     def run(cls, cost_fpath, features_fpath, capacity_class, resolution=128,
             xmission_config=None, sc_point_gids=None, nn_sinks=2,
-            clipping_buffer=1.05, barrier_mult=100, max_workers=None):
+            clipping_buffer=1.05, barrier_mult=100, max_workers=None,
+            save_paths=False, radius=None, simplify_geo=None):
         """
         Find Least Cost Tranmission connections between desired sc_points to
         given tranmission features for desired capacity class
@@ -561,10 +708,17 @@ class LeastCostXmission(LeastCostPaths):
         max_workers : int, optional
             Number of workers to use for processing, if 1 run in serial,
             if None use all available cores, by default None
+        save_paths : bool, optional
+            Flag to return least costs path as a multi-line geometry,
+            by default False
+        radius : None | int, optional
+            Force clipping radius if set to an int
+        simplify_geo : float | None, optional
+            If float, simplify geometries using this value
 
         Returns
         -------
-        least_costs : pandas.DataFrame
+        least_costs : pandas.DataFrame | gpd.DataFrame
             Least cost connections between all supply curve points and the
             transmission features with the given capacity class that are within
             "nn_sink" nearest infinite sinks
@@ -577,7 +731,10 @@ class LeastCostXmission(LeastCostPaths):
                                             nn_sinks=nn_sinks,
                                             clipping_buffer=clipping_buffer,
                                             barrier_mult=barrier_mult,
-                                            max_workers=max_workers)
+                                            max_workers=max_workers,
+                                            save_paths=save_paths,
+                                            radius=radius,
+                                            simplify_geo=simplify_geo)
 
         logger.info('{} connections were made to {} SC points in {:.4f} '
                     'minutes'
