@@ -41,6 +41,7 @@ class PlexosAggregation(BaseProfileAggregation):
     def __init__(self, plexos_nodes, rev_sc, reeds_build, cf_fpath,
                  forecast_fpath=None, build_year=2050, plexos_columns=None,
                  force_full_build=False, force_shape_map=False,
+                 plant_name_col=None, tech_tag=None, timezone='UTC',
                  max_workers=None):
         """
         Parameters
@@ -66,8 +67,11 @@ class PlexosAggregation(BaseProfileAggregation):
             File path to capacity factor file (reV gen output) to
             get profiles from.
         forecast_fpath : str | None
-            Forecasted capacity factor .h5 file path (reV results).
-            If not None, the generation profiles are sourced from this file.
+            Forecasted capacity factor .h5 file path (reV results). If not
+            None, the supply curve res_gids are mapped to sites in the
+            cf_fpath, then the coordinates from cf_fpath are mapped to the
+            nearest neighbor sites in the forecast_fpath, where the final
+            generation profiles are retrieved from.
         build_year : int, optional
             REEDS year of interest, by default 2050
         plexos_columns : list | None
@@ -81,6 +85,17 @@ class PlexosAggregation(BaseProfileAggregation):
             Flag to force the mapping of supply curve points to the plexos
             node shape file input (if a shape file is input) via nearest
             neighbor to shape centroid.
+        plant_name_col : str | None
+            Column in plexos_table that has the plant name that should be used
+            in the plexos output csv column headers.
+        tech_tag : str | None
+            Optional technology tag to include as a suffix in the plexos output
+            csv column headers.
+        timezone : str
+            Timezone for output generation profiles. This is a string that will
+            be passed to pytz.timezone() e.g. US/Pacific, US/Mountain,
+            US/Central, US/Eastern, or UTC. For a list of all available
+            timezones, see pytz.all_timezones
         max_workers : int | None
             Max workers for parallel profile aggregation. None uses all
             available workers. 1 will run in serial.
@@ -96,6 +111,9 @@ class PlexosAggregation(BaseProfileAggregation):
         self._force_full_build = force_full_build
         self._force_shape_map = force_shape_map
         self.max_workers = max_workers
+        self._plant_name_col = plant_name_col
+        self._tech_tag = tech_tag
+        self._timezone = timezone
 
         if plexos_columns is None:
             plexos_columns = tuple()
@@ -629,7 +647,9 @@ class PlexosAggregation(BaseProfileAggregation):
     @classmethod
     def run(cls, plexos_nodes, rev_sc, reeds_build, cf_fpath,
             forecast_fpath=None, build_year=2050, plexos_columns=None,
-            force_full_build=False, force_shape_map=False, max_workers=None):
+            force_full_build=False, force_shape_map=False,
+            plant_name_col=None, tech_tag=None, timezone='UTC',
+            out_fpath=None, max_workers=None):
         """Run plexos aggregation.
 
         Parameters
@@ -641,7 +661,9 @@ class PlexosAggregation(BaseProfileAggregation):
         rev_sc : str | pd.DataFrame
             reV supply curve results table including SC gid, latitude,
             longitude, res_gids, gid_counts. Or file path to reV supply
-            curve table.
+            curve table. Note that the gen_gids column in the rev_sc is ignored
+            and only the res_gids from rev_sc are mapped to the corresponding
+            "gid" column in the cf_fpath meta data.
         reeds_build : pd.DataFrame
             ReEDS buildout with rows for built capacity (MW) at each reV SC
             point. This should have columns: reeds_year, built_capacity, and
@@ -655,8 +677,11 @@ class PlexosAggregation(BaseProfileAggregation):
             File path to capacity factor file (reV gen output) to
             get profiles from.
         forecast_fpath : str | None
-            Forecasted capacity factor .h5 file path (reV results).
-            If not None, the generation profiles are sourced from this file.
+            Forecasted capacity factor .h5 file path (reV results). If not
+            None, the supply curve res_gids are mapped to sites in the
+            cf_fpath, then the coordinates from cf_fpath are mapped to the
+            nearest neighbor sites in the forecast_fpath, where the final
+            generation profiles are retrieved from.
         build_year : int
             REEDS year of interest.
         plexos_columns : list | None
@@ -670,6 +695,21 @@ class PlexosAggregation(BaseProfileAggregation):
             Flag to force the mapping of supply curve points to the plexos
             node shape file input (if a shape file is input) via nearest
             neighbor to shape centroid.
+        plant_name_col : str | None
+            Column in plexos_table that has the plant name that should be used
+            in the plexos output csv column headers.
+        tech_tag : str | None
+            Optional technology tag to include as a suffix in the plexos output
+            csv column headers.
+        timezone : str
+            Timezone for output generation profiles. This is a string that will
+            be passed to pytz.timezone() e.g. US/Pacific, US/Mountain,
+            US/Central, US/Eastern, or UTC. For a list of all available
+            timezones, see pytz.all_timezones
+        out_fpath : str, optional
+            Path to .h5 file into which plant buildout should be saved. A
+            plexos-formatted csv will also be written in the same directory.
+            By default None.
         max_workers : int | None
             Max workers for parallel profile aggregation. None uses all
             available workers. 1 will run in serial.
@@ -690,10 +730,16 @@ class PlexosAggregation(BaseProfileAggregation):
                  plexos_columns=plexos_columns,
                  force_full_build=force_full_build,
                  force_shape_map=force_shape_map,
+                 plant_name_col=plant_name_col,
+                 tech_tag=tech_tag,
+                 timezone=timezone,
                  max_workers=max_workers)
 
         profiles = pa.make_profiles()
 
+        if out_fpath is not None:
+            pa.export(pa.plexos_meta, pa.time_index, profiles, out_fpath)
+
         return pa.plexos_meta, pa.time_index, profiles
 
 
@@ -724,8 +770,11 @@ class RevReedsPlexosManager:
             File path to capacity factor file (reV gen output) to
             get profiles from.
         forecast_fpath : str | None
-            Forecasted capacity factor .h5 file path (reV results).
-            If not None, the generation profiles are sourced from this file.
+            Forecasted capacity factor .h5 file path (reV results). If not
+            None, the supply curve res_gids are mapped to sites in the
+            cf_fpath, then the coordinates from cf_fpath are mapped to the
+            nearest neighbor sites in the forecast_fpath, where the final
+            generation profiles are retrieved from.
         wait : int
             Integer seconds to wait for DB connection to become available
             before raising exception.
@@ -800,8 +849,11 @@ class RevReedsPlexosManager:
             File path to capacity factor file (reV gen output) to
             get profiles from.
         forecast_fpath : str | None
-            Forecasted capacity factor .h5 file path (reV results).
-            If not None, the generation profiles are sourced from this file.
+            Forecasted capacity factor .h5 file path (reV results). If not
+            None, the supply curve res_gids are mapped to sites in the
+            cf_fpath, then the coordinates from cf_fpath are mapped to the
+            nearest neighbor sites in the forecast_fpath, where the final
+            generation profiles are retrieved from.
         agg_kwargs : dict
             Optional additional kwargs for the aggregation run.
         wait : int
